package com.qiaoba.common.base.context;

import java.util.HashMap;
import java.util.Map;

/**
 * 全局上下文对象
 *
 * @author ailanyin
 * @version 1.0
 * @since 2023/5/22 17:24
 */
public class BaseContext {

    private static final String DATABASE_TYPE_MAP_KEY = "databaseType";

    private static final ThreadLocal<Map<String, Object>> CONTEXT_HOLDER = new ThreadLocal<>();

    /**
     * 获取上下文中的数据源
     */
    public String getDatabaseType() {
        return CONTEXT_HOLDER.get().get(DATABASE_TYPE_MAP_KEY).toString();
    }

    /**
     * 设置上下文中的数据源
     */
    public void setDatabaseType(String type) {
        Map<String, Object> map = new HashMap<>(1);
        map.put(DATABASE_TYPE_MAP_KEY, type);
        CONTEXT_HOLDER.set(map);
    }

    /**
     * 清除上下文中的数据源
     */
    public void clearDataSource() {
        CONTEXT_HOLDER.remove();
    }


}
