package com.qiaoba.common.database.utils;

import cn.hutool.core.convert.Convert;
import com.qiaoba.common.base.exceptions.ServiceException;
import com.qiaoba.common.database.config.DynamicDataSourceConfig;
import com.qiaoba.common.database.enums.DataBaseTypeEnum;
import lombok.extern.slf4j.Slf4j;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.SQLException;

/**
 * 数据库工具类
 *
 * @author ailanyin
 * @version 1.0
 * @since 2023/5/22 16:41
 */
@Slf4j
public class DatabaseUtil {


    /**
     * 获取当前数据库类型
     */
    public static DataBaseTypeEnum getDataBaseType() {
        DataSource dataSource = (DataSource) DynamicDataSourceConfig.DATA_SOURCE_MAP.get("");
        try (Connection conn = dataSource.getConnection()) {
            DatabaseMetaData metaData = conn.getMetaData();
            String databaseProductName = metaData.getDatabaseProductName();
            return DataBaseTypeEnum.find(databaseProductName);
        } catch (SQLException e) {
            throw new ServiceException(e.getMessage());
        }
    }

    public static String handleFindInSet(Object var1, String var2) {
        DataBaseTypeEnum dataBaseType = getDataBaseType();
        String var = Convert.toStr(var1);
        if (dataBaseType == DataBaseTypeEnum.SQL_SERVER) {
            // charindex(',100,' , ',0,100,101,') <> 0
            return "charindex('," + var + ",' , ','+" + var2 + "+',') <> 0";
        } else if (dataBaseType == DataBaseTypeEnum.POSTGRE_SQL) {
            // (select position(',100,' in ',0,100,101,')) <> 0
            return "(select position('," + var + ",' in ','||" + var2 + "||',')) <> 0";
        } else if (dataBaseType == DataBaseTypeEnum.ORACLE) {
            // instr(',0,100,101,' , ',100,') <> 0
            return "instr(','||" + var2 + "||',' , '," + var + ",') <> 0";
        }
        // find_in_set(100 , '0,100,101')
        return "find_in_set(" + var + " , " + var2 + ") <> 0";
    }
}
